/*
    FreeRTOS V6.0.0 - Copyright (C) 2009 Real Time Engineers Ltd.

    ***************************************************************************
    *                                                                         *
    * If you are:                                                             *
    *                                                                         *
    *    + New to FreeRTOS,                                                   *
    *    + Wanting to learn FreeRTOS or multitasking in general quickly       *
    *    + Looking for basic training,                                        *
    *    + Wanting to improve your FreeRTOS skills and productivity           *
    *                                                                         *
    * then take a look at the FreeRTOS eBook                                  *
    *                                                                         *
    *        "Using the FreeRTOS Real Time Kernel - a Practical Guide"        *
    *                  http://www.FreeRTOS.org/Documentation                  *
    *                                                                         *
    * A pdf reference manual is also available.  Both are usually delivered   *
    * to your inbox within 20 minutes to two hours when purchased between 8am *
    * and 8pm GMT (although please allow up to 24 hours in case of            *
    * exceptional circumstances).  Thank you for your support!                *
    *                                                                         *
    ***************************************************************************

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public 
    License and the FreeRTOS license exception along with FreeRTOS; if not it 
    can be viewed here: http://www.freertos.org/a00114.html and also obtained 
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/*
 * Creates all the demo application tasks, then starts the scheduler.  The WEB
 * documentation provides more details of the demo application tasks.
 * 
 * Main. c also creates a task called "Check".  This only executes every three 
 * seconds but has the highest priority so is guaranteed to get processor time.  
 * Its main function is to check that all the other tasks are still operational.  
 * Each task that does not flash an LED maintains a unique count that is 
 * incremented each time the task successfully completes its function.  Should 
 * any error occur within such a task the count is permanently halted.  The 
 * check task inspects the count of each task to ensure it has changed since
 * the last time the check task executed.  If all the count variables have 
 * changed all the tasks are still executing error free, and the check task
 * toggles an LED.  Should any task contain an error at any time the LED toggle
 * will stop.
 *
 * The LED flash and communications test tasks do not maintain a count.
 */

/*
Changes from V1.2.0
	
	+ Changed the baud rate for the serial test from 19200 to 57600.

Changes from V1.2.3

	+ The integer and comtest tasks are now used when the cooperative scheduler 
	  is being used.  Previously they were only used with the preemptive
	  scheduler.

Changes from V1.2.5

	+ Set the baud rate to 38400.  This has a smaller error percentage with an
	  8MHz clock (according to the manual).

Changes from V2.0.0

	+ Delay periods are now specified using variables and constants of
	  portTickType rather than unsigned long.

Changes from V2.6.1

	+ The IAR and WinAVR AVR ports are now maintained separately.

Changes from V4.0.5

	+ Modified to demonstrate the use of co-routines.

*/

#include <stdlib.h>
#include <string.h>

#ifdef GCC_MEGA_AVR
	/* EEPROM routines used only with the WinAVR compiler. */
	#include <avr/eeprom.h> 
#endif
	#include <avr/wdt.h>

/* Scheduler include files. */
#include "FreeRTOS.h"
#include "task.h"
//#include "croutine.h"

/* Demo file headers. */
/*
#include "PollQ.h"
#include "integer.h"
#include "serial.h"
#include "comtest.h"
#include "crflash.h"
#include "print.h"
#include "partest.h"
#include "regtest.h"
*/

/* Priority definitions for most of the tasks in the demo application.  Some
tasks just use the idle priority. */
#define mainLED_TASK_PRIORITY			( tskIDLE_PRIORITY + 1 )
#define mainCOM_TEST_PRIORITY			( tskIDLE_PRIORITY + 2 )
#define mainQUEUE_POLL_PRIORITY			( tskIDLE_PRIORITY + 2 )
#define mainCHECK_TASK_PRIORITY			( tskIDLE_PRIORITY + 3 )

/* Baud rate used by the serial port tasks. */
#define mainCOM_TEST_BAUD_RATE			( ( unsigned long ) 38400 )

/* LED used by the serial port tasks.  This is toggled on each character Tx,
and mainCOM_TEST_LED + 1 is toggles on each character Rx. */
#define mainCOM_TEST_LED				( 4 )

/* LED that is toggled by the check task.  The check task periodically checks
that all the other tasks are operating without error.  If no errors are found
the LED is toggled.  If an error is found at any time the LED is never toggles
again. */
#define mainCHECK_TASK_LED				( 6 )

/* The period between executions of the check task. */
#define mainCHECK_PERIOD				( ( portTickType ) 3000 / portTICK_RATE_MS  )

/* An address in the EEPROM used to count resets.  This is used to check that
the demo application is not unexpectedly resetting. */
#define mainRESET_COUNT_ADDRESS			( ( void * ) 0x50 )

/* The number of coroutines to create. */
#define mainNUM_FLASH_COROUTINES		( 3 )

/* Task Handles */
xTaskHandle hToggleMyLed1, hToggleMyLed2, hToggleMyLed3, hToggleMyLed4, hToggleMyLed5, hToggleMyLed6, hToggleMyLed7, hToggleMyLed8;
xTaskHandle hButtonOnTask, hButtonOffTask;
xTaskHandle hADCTask;

/*
 * The task function for the "Check" task.
 */
static void vErrorChecks( void *pvParameters );

/*
 * Checks the unique counts of other tasks to ensure they are still operational.
 * Flashes an LED if everything is okay. 
 */
static void prvCheckOtherTasksAreStillRunning( void );

/*
 * Called on boot to increment a count stored in the EEPROM.  This is used to 
 * ensure the CPU does not reset unexpectedly.
 */
static void prvIncrementResetCount( void );

/*
 * The idle hook is used to scheduler co-routines.
 */
void vApplicationIdleHook( void );

/*
 * Bzzt.
 */
void vInitClock( void );

/*
 * LED toggeling tasks:
 */
static void vToggleLED1( void *pvParameters );
static void vToggleLED2( void *pvParameters );
static void vToggleLED3( void *pvParameters );
static void vToggleLED4( void *pvParameters );
static void vToggleLED5( void *pvParameters );
static void vToggleLED6( void *pvParameters );
static void vToggleLED7( void *pvParameters );
static void vToggleLED8( void *pvParameters );

static void vButtonOnTask( void *pvParameters );
static void vButtonOffTask( void *pvParameters );
static void vADCTask( void *pvParameters );

/*-----------------------------------------------------------*/

int main( void )
{
	
	//init clock
    vInitClock();
	//Init port settings
	PORTCFG.MPCMASK=0xFF;	// set the mask for pin configurations, all pins are masked 
	PORTF.PIN0CTRL=0x38;	// activate a pullup for all masked pins
	PORTF.DIR=0x00;			// set port F as input
	PORTE.DIR=0xFF;			// set port E as output
    PORTE.OUT=0xFF;         // set all LEDS on Port E to OFF state



	/* Create the standard demo tasks. */
	
	/* Create the tasks defined within this file. */
	xTaskCreate( vToggleLED1 , ( signed char * ) "ToggleMyLed1", configMINIMAL_STACK_SIZE, NULL, (4), &hToggleMyLed1 );
	xTaskCreate( vToggleLED2 , ( signed char * ) "ToggleMyLed2", configMINIMAL_STACK_SIZE, NULL, (4), &hToggleMyLed2 );
	xTaskCreate( vToggleLED3 , ( signed char * ) "ToggleMyLed3", configMINIMAL_STACK_SIZE, NULL, (4), &hToggleMyLed3 );
	xTaskCreate( vToggleLED4 , ( signed char * ) "ToggleMyLed4", configMINIMAL_STACK_SIZE, NULL, (4), &hToggleMyLed4 );
	xTaskCreate( vToggleLED5 , ( signed char * ) "ToggleMyLed5", configMINIMAL_STACK_SIZE, NULL, (4), &hToggleMyLed5 );
	xTaskCreate( vToggleLED6 , ( signed char * ) "ToggleMyLed6", configMINIMAL_STACK_SIZE, NULL, (4), &hToggleMyLed6 );
	xTaskCreate( vToggleLED7 , ( signed char * ) "ToggleMyLed7", configMINIMAL_STACK_SIZE, NULL, (4), &hToggleMyLed7 );
	xTaskCreate( vToggleLED8 , ( signed char * ) "ToggleMyLed8", configMINIMAL_STACK_SIZE, NULL, (4), &hToggleMyLed8 );
	xTaskCreate( vButtonOnTask, ( signed char * ) "ButtonOnTask", configMINIMAL_STACK_SIZE, NULL, (5), &hButtonOnTask );
	xTaskCreate( vButtonOffTask, ( signed char * ) "ButtonOffTask", configMINIMAL_STACK_SIZE, NULL, (5), &hButtonOffTask );
	xTaskCreate( vADCTask, ( signed char * ) "ADCTask", configMINIMAL_STACK_SIZE, NULL, (4), &hADCTask );


	/* In this port, to use preemptive scheduler define configUSE_PREEMPTION 
	as 1 in portmacro.h.  To use the cooperative scheduler define 
	configUSE_PREEMPTION as 0. */
	vTaskStartScheduler();

	return 0;
}
/*-----------------------------------------------------------*/

static void vErrorChecks( void *pvParameters )
{
static volatile unsigned long ulDummyVariable = 3UL;

	/* The parameters are not used. */
	( void ) pvParameters;

	/* Cycle for ever, delaying then checking all the other tasks are still
	operating without error. */
	for( ;; )
	{
		vTaskDelay( mainCHECK_PERIOD );

		/* Perform a bit of 32bit maths to ensure the registers used by the 
		integer tasks get some exercise. The result here is not important - 
		see the demo application documentation for more info. */
		ulDummyVariable *= 3;
		
		prvCheckOtherTasksAreStillRunning();
	}
}
/*-----------------------------------------------------------*/

static void prvCheckOtherTasksAreStillRunning( void )
{
static portBASE_TYPE xErrorHasOccurred = pdFALSE;

	if( xAreIntegerMathsTaskStillRunning() != pdTRUE )
	{
		xErrorHasOccurred = pdTRUE;
	}

	if( xAreComTestTasksStillRunning() != pdTRUE )
	{
		xErrorHasOccurred = pdTRUE;
	}

	if( xArePollingQueuesStillRunning() != pdTRUE )
	{
		xErrorHasOccurred = pdTRUE;
	}

	if( xAreRegTestTasksStillRunning() != pdTRUE )
	{
		xErrorHasOccurred = pdTRUE;
	}
	
	if( xErrorHasOccurred == pdFALSE )
	{
		/* Toggle the LED if everything is okay so we know if an error occurs even if not
		using console IO. */
		vParTestToggleLED( mainCHECK_TASK_LED );
	}
}
/*-----------------------------------------------------------*/

static void prvIncrementResetCount( void )
{
unsigned char ucCount;

	eeprom_read_block( &ucCount, mainRESET_COUNT_ADDRESS, sizeof( ucCount ) );
	ucCount++;
	eeprom_write_byte( mainRESET_COUNT_ADDRESS, ucCount );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	//vCoRoutineSchedule();
}

/*
void vApplicationTickHook( void )
{
	    //wdt_reset();
}
*/



static void vToggleLED1( void *pvParameters )
{
	//suspend the adc, so it is not influencing the leds all the time
	vTaskSuspend(hADCTask);
	for( ;; )
	{
		
			vTaskDelay(500);
			PORTE.OUT ^= 0x01;
	}

}

static void vToggleLED2( void *pvParameters )
{
	for( ;; )
	{
		
			vTaskDelay(501);
			PORTE.OUT ^= 0x02;
	}

}

static void vToggleLED3( void *pvParameters )
{
	for( ;; )
	{				
		
			vTaskDelay(502);
			PORTE.OUT ^= 0x04;
	}

}

static void vToggleLED4( void *pvParameters )
{
	for( ;; )
	{				
		
			vTaskDelay(503);
			PORTE.OUT ^= 0x08;
	}

}

static void vToggleLED5( void *pvParameters )
{
	for( ;; )
	{				
		
			vTaskDelay(504);
			PORTE.OUT ^= 0x10;
	}

}

static void vToggleLED6( void *pvParameters )
{
	for( ;; )
	{				
		
			vTaskDelay(505);
			PORTE.OUT ^= 0x20;
	}

}

static void vToggleLED7( void *pvParameters )
{
	for( ;; )
	{				
		
			vTaskDelay(506);
			PORTE.OUT ^= 0x40;
	}

}

static void vToggleLED8( void *pvParameters )
{
	for( ;; )
	{				
		
			vTaskDelay(507);
			PORTE.OUT ^= 0x80;
	}

}

void vInitClock( void )
{
   /* put osc to 32Mhz PIN1 = OSC_RC32MEN */
   OSC.CTRL |= 0x02;
   /* wait until osc stability and set the flag RC32MRDY */
   while(!(OSC.STATUS & OSC_RC32MRDY_bm));
   /* I/O protection */
   CCP = 0xD8;
   /* set clock to 32MHz */
   CLK.CTRL = 0x01;
}

static void vButtonOffTask( void *pvParameters )
{
	for( ;; )
	{
		vTaskDelay(5);
		if(!(PORTF.IN & 0x01)) //if Button1 pressed and leds are not suspended
		{
			//enter a critical section
			taskENTER_CRITICAL();
    		vTaskDelay(250);
			//set LEDs to off
			PORTE.DIR = 0x00;
			PORTE.OUT = 0xFF;
			PORTE.DIR = 0xFF;
			//suspend LED tasks
			vTaskSuspend( hToggleMyLed1 );
			vTaskSuspend( hToggleMyLed2 );
			vTaskSuspend( hToggleMyLed3 );
			vTaskSuspend( hToggleMyLed4 );
			vTaskSuspend( hToggleMyLed5 );
			vTaskSuspend( hToggleMyLed6 );
			vTaskSuspend( hToggleMyLed7 );
			vTaskSuspend( hToggleMyLed8 );
			//resume  ADC task
			vTaskResume(hADCTask);
			taskEXIT_CRITICAL();

		}
	}

}

static void vButtonOnTask( void *pvParameters )
{
	for( ;; )
	{
		vTaskDelay(5);
		if(!(PORTF.IN & 0x02)) //if button2 pressed and leds are suspended
		{
			//enter a critical section
			taskENTER_CRITICAL();
    		vTaskDelay(250);
			//set LEDs to off
            //
			PORTE.DIR = 0x00;
			PORTE.OUT = 0xFF;
			PORTE.DIR = 0xFF;
			//resume LED tasks
			vTaskResume( hToggleMyLed8 );
			vTaskResume( hToggleMyLed7 );
			vTaskResume( hToggleMyLed6 );
			vTaskResume( hToggleMyLed5 );
			vTaskResume( hToggleMyLed4 );
			vTaskResume( hToggleMyLed3 );
			vTaskResume( hToggleMyLed2 );
			vTaskResume( hToggleMyLed1 );
			//suspend  ADC task
			vTaskSuspend(hADCTask);
			taskEXIT_CRITICAL();
		}
	}

}

static void vADCTask( void *pvParameters )
{
	for( ;; )
	{				
		//this task is a dummy,
		//which just toggels
		//a LED and should become an ADC Task
		//later on
		vTaskDelay(500);
		PORTE.OUT ^= 0x80;
	}
}
